package api

import (
	"3dshow-supplier/conf"
	"3dshow-supplier/db/model"
	"3dshow-supplier/db/repo"
	"context"
	"errors"
	"fmt"
	"image"
	"os"
	"os/exec"
	"path"
	"strings"
	"time"

	_ "image/jpeg"
	_ "image/png"

	"github.com/gin-gonic/gin"
	"github.com/huaweicloud/huaweicloud-sdk-go-obs/obs"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

var ffmpegExe = ""

func regAssetApi(r *GinRouter) {

	// 添加管理端接口
	CreateCRUD(r, "/assets", &CRUDOption{
		Collection: repo.CollectionAssets,
		NewModel: func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
			entity := &model.Asset360Fake3d{}
			c.ShouldBindJSON(entity)

			entity.SupplyId, _ = primitive.ObjectIDFromHex(apictx.User.Parent)
			entity.CreateTime = time.Now()

			return entity, nil
		},

		EmtyModel: func(c *gin.Context, apictx *ApiSession) interface{} {
			return &model.Asset360Fake3d{}
		},
		SearchSort: bson.M{"createTime": -1},
		SearchFilter: func(c *gin.Context, apictx *ApiSession, query map[string]interface{}) map[string]interface{} {
			if query["supplyId"] != nil {
				query["supplyId"], _ = primitive.ObjectIDFromHex(query["supplyId"].(string))
			}
			return query
		},
		JWT:           true,
		SearchProject: []string{"name", "createTime", "type", "unit", "price", "cover", "status"},
	})

	r.POST("/assets/conv/:id/:groupId", AssetsConv)
}

func AssetsConv(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	params := c.Params
	id, ok := params.Get("id")
	if !ok {
		return nil, errors.New("id不能为空")
	}
	groupId, ok := params.Get("groupId")
	if !ok {
		return nil, errors.New("groupId不能为空")
	}

	startTime := time.Now()
	dir, _ := os.Getwd()
	ffmpegExe = fmt.Sprintf("%s/ffmpeg", dir)
	wPath := fmt.Sprintf("%s/%s/%s/spin_360", "assets", id, groupId)
	oPath := fmt.Sprintf("%s/%s/%s/spin_360", "images", id, groupId)

	// todo下载图片到oPth

	// 获取图片
	files, _ := os.ReadDir(oPath)
	levels := map[int]int{4: 10, 2: 11, 1: 12}

	// 递归创建
	os.MkdirAll(wPath, os.ModePerm)

	for _, f := range files {
		fi, _ := os.Open(fmt.Sprintf("./%s/%s", oPath, f.Name()))

		imageInfo, _, _ := image.DecodeConfig(fi)
		defer fi.Close()

		filePrefix := getFilePrefix(f.Name())
		// 生成每个文件目录 00_00_files
		preFileDir := fmt.Sprintf("%s/%s%s", wPath, filePrefix, "_files")
		os.Mkdir(preFileDir, os.ModePerm)

		// 生成cover 384/512
		// asset/spin_360/00_00_cover.png
		orignFile := fmt.Sprintf("%s/%s", oPath, f.Name())
		coverFile := fmt.Sprintf("%s/%s_cover%s", wPath, filePrefix, path.Ext(f.Name()))
		scaleImage(384, 512, orignFile, coverFile)

		// 生成json文件 asset/spin_360/00_00.json
		jsonContent := fmt.Sprintf("{\"tileSource\": \"%s\",\"tileSize\": 510,\"width\": 1944,\"height\": 2592,\"sceneType\": \"spin\",\"scaleType\": \"inside\",\"minLevel\": 10,\"coverWidth\": 384,\"coverHeight\": 512,\"dziFile\": \"%s.dzi\",\"coverImage\":\"%s\"}", fmt.Sprintf("%s%s", filePrefix, "_files"), filePrefix, fmt.Sprintf("%s_cover%s", filePrefix, path.Ext(f.Name())))
		jsonFileName := fmt.Sprintf("%s/%s.json", wPath, filePrefix)
		setJsonFile(jsonFileName, jsonContent)

		// 文件目录下level目录 00_00_files/10 11 23
		for scale, level := range levels {
			preLevelDir := fmt.Sprintf("%s/%s%s/%d", wPath, filePrefix, "_files", level)
			os.Mkdir(preLevelDir, os.ModePerm)

			// 生成缩略图
			levelW := imageInfo.Width / scale
			levelH := imageInfo.Height / scale

			levelDest := fmt.Sprintf("%s/%s%s/%d", wPath, filePrefix, "_files", level)
			levelScaleImage := fmt.Sprintf("%s/%s_scale%s", levelDest, filePrefix, path.Ext(f.Name()))
			scaleImage(levelW, levelH, orignFile, levelScaleImage)

			// 裁剪缩略图
			createLevel(levelW, levelH, levelScaleImage, levelDest)
			os.Remove(levelScaleImage)
		}
	}
	// 删除图片文件
	// os.Remove(oPath)
	// os.Remove(wPath)

	fmt.Println(time.Since(startTime).Seconds())

	key := wPath
	_url, _ := serviceObsUploadPolicy(key, apictx.Svc.Conf)

	url := fmt.Sprintf("%s%s", "https://www.3dqueen.cloud/asset360/index.html?apath=", _url)

	result, err := repo.RepoDocArrayOneUpdate(apictx.CreateRepoCtx(), &repo.ArrayOneUpdateOption{
		CollectName: repo.CollectionAssets,
		Id:          id,
		Query:       repo.Map{"groups.id": groupId},
		Set:         repo.Map{"groups.$.url": url},
	})

	return result, err

}

func createLevel(w, h int, source, dest string) {

	xIndex := 0
	yIndex := 0
	step := 510
	for {

		offsetX := xIndex * step
		cropX := offsetX - 1
		cropW := 512
		if xIndex == 0 {
			cropW = 511
			cropX = 0
		}
		isOver := false
		if w < 511 {
			cropW = w
			isOver = true
		}
		if (offsetX - 1 + cropW) >= w {
			cropW = w - offsetX + 1
			isOver = true
		}

		for {

			offsetY := yIndex * step
			cropY := offsetY - 1
			cropH := 512
			if yIndex == 0 {
				cropH = 511
				cropY = 0
			}
			isYOver := false
			if (offsetY - 1 + cropH) >= h {
				cropH = h - offsetY + 1
				isYOver = true
			}

			//开始按参数裁剪
			cropImage(cropX, cropY, cropW, cropH, xIndex, yIndex, source, dest)
			fmt.Println("x:", cropX)
			fmt.Println("y:", cropY)
			fmt.Println("w:", cropW)
			fmt.Println("h:", cropH)
			fmt.Println("yIndex:", yIndex)

			yIndex = yIndex + 1

			if isYOver {
				break
			}
		}

		yIndex = 0
		xIndex = xIndex + 1
		if isOver {
			break
		}
	}
}

// asset/spin_360/00_00.json
func setJsonFile(fileName string, content string) {
	f, err := os.Create(fileName)
	if err != nil {
		return
	}
	f.WriteString(content)
	defer f.Close()
}

// 获取不带后缀文件名
func getFilePrefix(fileName string) string {
	fileSilce := strings.Split(fileName, ".")
	return fileSilce[0]
}

// asset/spin_360_scene/00_00_files/10/00_00_scale.png asset/spin_360_scene/00_00_files/10
func cropImage(x, y, w, h int, xIndex, yIndex int, source, dest string) {
	ctx, cancel := context.WithTimeout(context.Background(), time.Duration(50000)*time.Millisecond)
	cmd := exec.CommandContext(ctx, ffmpegExe,
		"-i", source,
		"-vf", fmt.Sprintf("crop=%d:%d:%d:%d", w, h, x, y),
		fmt.Sprintf("%s/%d_%d%s", dest, xIndex, yIndex, path.Ext(source)),
	)
	defer cancel()

	err := cmd.Run()
	if err != nil {
		errImage := fmt.Sprintf("%s/%d_%d%s", dest, xIndex, yIndex, path.Ext(source))
		fmt.Println("err image:", errImage)
		fmt.Println("crop err", err)
	}
}

func scaleImage(w, h int, source, dest string) {
	ctx, cancel := context.WithTimeout(context.Background(), time.Duration(50000)*time.Millisecond)
	// ffmpeg -i image_source -vf scale=width:height out_source
	cmd := exec.CommandContext(ctx, ffmpegExe,
		"-i", source,
		"-vf", fmt.Sprintf("scale=%d:%d", w, h),
		dest,
	)
	defer cancel()

	err := cmd.Run()
	if err != nil {
		fmt.Println("scale err", err)
	}
}

func serviceObsUploadPolicy(key string, conf *conf.AppConf) (interface{}, error) {

	client, err := CreateObsClient()
	if err != nil {
		return nil, NewError("创建ObsClient 失败!")
	}
	defer func() {
		client.Close()
	}()
	bucketName := conf.Obs.Bucket
	_, err = client.CreateBrowserBasedSignature(&obs.CreateBrowserBasedSignatureInput{
		Bucket:  bucketName,
		Key:     key,
		Expires: 600,
		FormParams: map[string]string{
			"x-obs-acl": "public-read",
		},
	})

	if err != nil {
		return nil, NewLogWithError(err)
	}
	obsConf := conf.Obs
	// out := map[string]interface{}{
	// 	"accessKeyId":  obsConf.AccessKeyId,
	// 	"originPolicy": result.OriginPolicy,
	// 	"policy":       result.Policy,
	// 	"signature":    result.Signature,
	// 	"host":         fmt.Sprintf("https://%s.%s", bucketName, obsConf.Endpoint),
	// 	"key":          key,
	// 	"saveType":     conf.SaveType,
	// }
	url := fmt.Sprintf("%s/%s", fmt.Sprintf("https://%s.%s", bucketName, obsConf.Endpoint), key)
	return url, nil
}

func CreateObsClient() (*obs.ObsClient, error) {
	obsConf := conf.AppConfig.Obs
	return obs.New(obsConf.AccessKeyId, obsConf.SecrateKey, obsConf.Endpoint)
}
